 using System;
 using System.Collections.Generic;
 using System.Linq;
 using System.Text;
 using System.Threading.Tasks;
 
 using SharpCharts.Base.Chart;
 using System.ComponentModel;
 using System.Xml.Serialization;
 using SharpCharts.Base.SharpScript;
 
 namespace SharpCharts.Base.Data
 {
     public class Footprint : BarType
     {
     	FootprintBars footprintBars;
     	double lastPrice = double.MinValue;
     	
     	public Footprint() 
     		: this(30, 2)
     	{ }
     	
     	public Footprint(int barValue, double ticksize)
     	{
     		this.Items = new List<FootprintBars>();
     		this.TickSize = ticksize;
     		this.BarValue = barValue;
     	}
     
         public override void Add(Bars bars, DateTime time, double open, double high, double low, double close, double bid, double ask, long volume, long openInterest, bool isRealTime)
 		{
 			
 			if (!isRealTime)
 			{
 				if (bid <= 0.0d && ask <= 0.0d)
 				{
 					if (this.lastPrice == double.MinValue)
 					{
 						this.lastPrice = close;
						return; 					
 					}
 				
 					if (close > this.lastPrice)
 					{
 						bid = this.lastPrice;
 						ask = close;
 					}
 					else if (close < this.lastPrice)
 					{
 						bid = close;
 						ask = this.lastPrice;
 					}
 					else
 					{
 						bid = ask = close;
 					}
 				
 					this.lastPrice = close;
 				}
 				else
 				{
 					this.lastPrice = double.MinValue;
 				}
 			}
 			
             if (bars.Count == 0)
             {
             	this.BasePrice = (int)(close / Math.Max(this.TickSize, bars.Instrument.InstrumentDefination.TickSize)) * Math.Max(this.TickSize, bars.Instrument.InstrumentDefination.TickSize);
                 AddBar(bars, CalculateNewBarTime(bars, time), open, high, low, close, volume, openInterest, isRealTime);
                 this.footprintBars = new FootprintBars(this, Math.Max(this.TickSize, bars.Instrument.InstrumentDefination.TickSize));
                 this.footprintBars.AssignValues(close, bid, ask, volume);
                 this.Items.Add(footprintBars);
                 
                 this.MainBar = new FootprintBars(this.BasePrice, Math.Max(this.TickSize, bars.Instrument.InstrumentDefination.TickSize));
                 this.MainBar.AssignValues(close, bid, ask, volume);
             }
             else
             {
                 if (time > bars.GetBar(bars.Count - 1).Time)
                 {
                     AddBar(bars, CalculateNewBarTime(bars, time), open, high, low, close, volume, openInterest, isRealTime);
                     this.footprintBars = new FootprintBars(this, Math.Max(this.TickSize, bars.Instrument.InstrumentDefination.TickSize));
                     this.footprintBars.AssignValues(close, bid, ask, volume);
                     this.Items.Add(footprintBars);
                     if (this.MainBar != null)
                     {
                     	this.MainBar.AssignValues(close, bid, ask, volume);
                     }
                 }
                 else
                 {
                     
                     UpdateBar(bars, bars.GetBar(bars.Count - 1).Time, high, low, close, volume, openInterest, isRealTime);
 
                     if (this.footprintBars == null)
                     {
                         throw new NullReferenceException("Footprint.Add : profileItem is null");
                     }
 
                     this.footprintBars.AssignValues(close, bid, ask, volume);
                     if (this.MainBar != null)
                     {
                     	this.MainBar.AssignValues(close, bid, ask, volume);
                     }
 
                 }
             }
         }
         
         private DateTime CalculateNewBarTime(Bars bars, DateTime time)
         {
            TimeSpan ts = TimeSpan.FromMinutes(barValue);
            return new DateTime(((time.Ticks + ts.Ticks - 1) / ts.Ticks) * ts.Ticks);   
         }
         
		protected override void Clone(BarType clone)
		{
			Footprint fp = clone as Footprint;
			if (fp == null)
			{
				base.Clone(clone);
			}
			else
			{
				fp.BarValue = this.BarValue;
				fp.TickSize = this.TickSize;
			}
		}
         
 
         public override Chart.ChartStyle[] SupportedChartStyles
         {
             get { return new Chart.ChartStyle[] { new FootprintDelta() }; }
         }
 		
 		private int barValue = 30;
 		[SharpCharts.Base.SharpScript.Input]
         public override int BarValue
         {
             get
             {
                 return barValue;;
             }
             set
             {
                 barValue = value;
             }
         }
 
         public override BuiltDataType BuiltDataType
         {
             get { return BuiltDataType.Tick; }
         }
 
         public override bool IsIntraday
         {
             get { return true; }
         }
         
         [Input]
         public double TickSize { get; set; }
         
         [XmlIgnore]
 		 public List<FootprintBars> Items { get; private set; } 
 
 		[XmlIgnore]
 		public long MaxPOCVolume { get; set; }
 		
 		[XmlIgnore]
 		public double BasePrice { get; private set; }
 
 		[XmlIgnore]
 		public FootprintBars MainBar { get; private set; } 
     }
     
     
     public class FootprintItem
     {
     	public FootprintItem(double price)
     	{
     		this.Price = price;
     	}
     	
     	
     	public double Price { get; private set; }
     	public long BidVolume { get; set; }
     	public long AskVolume { get; set; }
     	public long NutralVolume {get; set; }
     	public long TotalVolume
     	{
     		get { return this.BidVolume + this.AskVolume + NutralVolume; }
     	}
     	public long Delta
     	{
     		get
     		{
     			return this.AskVolume - this.BidVolume;
     		}
     	}
     	
     	
     }
     
     
     public class FootprintBars
     {
         
         double tickSize = 0.05;
         
         double basePrice = double.MinValue;
         Footprint parent;
         
         public FootprintBars(Footprint parent, double tickSize) 
         	: this(tickSize)
         {
             this.basePrice = parent.BasePrice;
             this.parent = parent;
         }
         
         public FootprintBars(double basePrice, double tickSize) 
         	: this(tickSize)
         {
         	this.basePrice = basePrice;
         }
         
         private FootprintBars(double tickSize)
         {
         	 this.Max = double.MinValue;
             this.Min = double.MaxValue;
             
             this.Profiles = new long[0];
             
             this.tickSize = tickSize;
         }
 
         public double Max { get; private set; }
         public double Min { get; private set; }
         public double POC { get; private set; }
         
         
         public long POCVolume { get; private set; }
		 public long Delta
		 {
		 	get { return this.AskDelta - this.BidDelta; }
		 }
		 public long Nutral { get; private set; }
         public long AskDelta { get; set; }
         public long BidDelta { get; set; }
         public long TotalVolume 
         { 
         	get { return this.AskDelta + this.BidDelta + this.Nutral; }
         }
 		 
 		 public double PercentageAskDelta
 		 {
 		 	get { return Math.Round((double)this.AskDelta / (double)this.TotalVolume * 100, 2); }
 		 }
 		 
 		 public double PercentageBidDelta
 		 {
 		 	get { return Math.Round((double)this.BidDelta / (double)this.TotalVolume * 100, 2); }
 		 }
 		 
         public FootprintItem[] Bars { get; private set; }
         public long[] Profiles { get; set; }
         
         public double CalculateStepPrice(double value)
         {
         	if (Common.Globals.IsEqual(value, this.basePrice))
         		return value;
         		
         	if (value > this.basePrice)
         	{
         		return this.basePrice + (int)Math.Round((value - this.basePrice) / this.tickSize, MidpointRounding.AwayFromZero) * this.tickSize;
         	}
         	else
         	{
         		return this.basePrice - (int)Math.Round((this.basePrice - value) / this.tickSize, MidpointRounding.AwayFromZero) * this.tickSize;
         	}
         }
         
 			
         public void AssignValues(double price, double bidPrice, double askPrice, long volume)
         {
 			  			 
 			 double stepPrice = CalculateStepPrice(price);
 			 
             if (this.Bars == null)
             {             	
             	FootprintItem item = new FootprintItem(stepPrice);
             	
             	this.CalculateDelta(item, price, bidPrice, askPrice, volume);
             	
                this.Profiles = new long[] { volume };
                this.Bars = new FootprintItem[] { item };

				this.Max = this.Min = this.POC = stepPrice;
                this.POCVolume = volume;
                if (this.parent != null)
                {
	                this.parent.MaxPOCVolume = Math.Max(this.parent.MaxPOCVolume, this.POCVolume);
	            }
                return;
             }
             
             
 
             if (stepPrice > this.Max)
             {
                 int steps = (int)Math.Round((stepPrice - this.Max) / this.tickSize, MidpointRounding.AwayFromZero);
                 
                 if (steps == 0)
                 	throw new Exception("max zero");
                 	
                 //reconfigure the arrays
                 this.Profiles = AddUp<long>(this.Profiles, steps);
                 this.Bars = AddUp<FootprintItem>(this.Bars, steps);
                 //add the values to the array
                 this.Profiles[0] = volume;
                 FootprintItem item = new FootprintItem(stepPrice);
                 this.CalculateDelta(item, price, bidPrice, askPrice, volume);
                 
                 this.Bars[0] = item;
                 if (volume > this.POCVolume)
                 {
                     this.POCVolume = volume;
                     this.POC = stepPrice;
                     if (this.parent != null)
                     {
                     	this.parent.MaxPOCVolume = Math.Max(this.parent.MaxPOCVolume, this.POCVolume);
                     }
                 }
                 this.Max = stepPrice;
             }
             
             else if (stepPrice < this.Min)
             {
                 int steps = (int)Math.Round((this.Min - stepPrice) / this.tickSize, MidpointRounding.AwayFromZero);
                		
                 this.Profiles = AddUp<long>(this.Profiles, steps * -1);
                 this.Bars = AddUp<FootprintItem>(this.Bars, steps * -1);
                 
                 this.Profiles[this.Profiles.Length - 1] = volume;
                 FootprintItem item = new FootprintItem(stepPrice);
                 this.CalculateDelta(item, price, bidPrice, askPrice, volume);
                 
                 this.Bars[this.Bars.Length - 1] = item;
                 
                 if (volume > this.POCVolume)
                 {
                     this.POCVolume = volume;
                     this.POC = stepPrice;
                     if (this.parent != null)
                     {
                     	this.parent.MaxPOCVolume = Math.Max(this.parent.MaxPOCVolume, this.POCVolume);
                     }
                 }
                 
                 this.Min = stepPrice;
             }
             else
             {
             
                 int idx = (int)Math.Round((this.Max - stepPrice) / this.tickSize, MidpointRounding.AwayFromZero);
                 
                 if (idx >= 0 && idx < this.Profiles.Length)
                 {
                     this.Profiles[idx] += volume;
 
                     if (this.Profiles[idx] > this.POCVolume)
                     {
                         this.POCVolume = this.Profiles[idx];
                         this.POC = stepPrice;
                         if (this.parent != null)
                         {
                         	this.parent.MaxPOCVolume = Math.Max(this.parent.MaxPOCVolume, this.POCVolume);
                         }
                     }
                     
                     FootprintItem item = this.Bars[idx];
                     
                     if (item == null)
                     {
                     	this.Bars[idx] = item = new FootprintItem(stepPrice);
                     }
                     
                     this.CalculateDelta(item, price, bidPrice, askPrice, volume);
                     
                 }
                 else
                 {
                     throw new IndexOutOfRangeException(string.Format("Footprint.AssignValues : idx {0} out of range", idx));
                 }
                
             }
         }
         
         private void CalculateDelta(FootprintItem item, double price, double bidPrice, double askPrice, long volume)
         {
         	if (item == null)
         		return;
         	
         	if (Math.Abs(price - bidPrice) > Math.Abs(askPrice - price))	//traded @ask
            {
            	item.AskVolume += volume;
            	this.AskDelta += volume;
            }
            else if (Math.Abs(price - bidPrice) < Math.Abs(askPrice - price))	//traded at bid
            {
            	item.BidVolume += volume;
            	this.BidDelta += volume;
            }
            else
            {
           		this.Nutral += volume;
           		item.NutralVolume += volume;
            }
            
         }
 
         private T[] AddUp<T>(T[] array, int steps)
         {
             T[] tmp = new T[array.Length + Math.Abs(steps)];
 
             if (steps >= 0)  //price increase
             {
                 array.CopyTo(tmp, steps);
                 
             }
             else
             {
                 array.CopyTo(tmp, 0);
             }
             return tmp;
         }
 
     }
 }