using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Windows;
using System.Windows.Media;
using SharpCharts.Base.Data;
using System.Xml.Serialization;
using SharpCharts.Base.Common;

namespace SharpCharts.Base.Chart
{
    public class FootprintDelta : ChartStyle
    {
    	Footprint barType;
    
    	public FootprintDelta()
    	{
    		base.BarSpace = 5.0d;
            base.BarWidth = 100.0d;
            base.UpBarBrush = Brushes.Green;
            this.ShowPOC = true;
            this.ShowProfile = true;
            this.IsPercentage = true;
            this.ProfileBrush = Brushes.Gray;
    	}

	bool onlyOnce = false;
	
        public override void DrawBars(DrawingContext dc, double chartElementHeight, double chartElementWidth, Bars bars, int startIdx, int endIdx, double min, double max)
        {
            if (this.barType == null)
            {
                this.barType = bars.BarType as Footprint;
                if (barType == null)
                    throw new InvalidCastException(string.Format("FootprintChartStyle.DrawBars : could not cast BarType - {0}", bars.BarType));
            }

			
			//***** draw the bars
            double x = chartElementWidth - base.BarWidth / 2;
			double width = 0.0d;
			double height = 0.0d;
			
            for (int i = endIdx; i >= startIdx; i--)
            {

                // draw the profile ***********************************

				height = width = 0.0d;
				
                GlyphRun deltaTxt = Common.Globals.GetGlyphRun(barType.Items[i].Delta.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x, chartElementHeight), true, out width, out height);
				dc.DrawGlyphRun(barType.Items[i].Delta >= 0 ? base.UpBarBrush : base.DownBarBrush, deltaTxt);
                
                if (height == 0.0d)
                	return;
                
                deltaTxt = Common.Globals.GetGlyphRun(this.IsPercentage ? string.Format("{0}%", barType.Items[i].PercentageBidDelta) :  barType.Items[i].BidDelta.ToString(), 
                	Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x - this.BarWidth / 4, chartElementHeight - height), true);
				dc.DrawGlyphRun(base.UpWickBrush, deltaTxt);
				
                deltaTxt = Common.Globals.GetGlyphRun(this.IsPercentage ? string.Format("{0}%", barType.Items[i].PercentageAskDelta) : barType.Items[i].AskDelta.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x + this.BarWidth / 4, chartElementHeight - height), true);
				dc.DrawGlyphRun(base.UpWickBrush, deltaTxt);
				
				if (this.ShowPOC)
				{
					double pocY = CalculateYPoint(min, max, barType.Items[i].POC, chartElementHeight) - height / 2;
					if (pocY > 0 && pocY < chartElementHeight - height * 3)
					{
						dc.DrawRectangle(null, base.UpWickPen, new Rect(x - this.BarWidth / 2, pocY, base.BarWidth, height));
					}
				}
                
                //draw the candles
                IBar bar = bars.GetBar(i);

                double highY = base.CalculateYPoint(min, max, bar.High, chartElementHeight);
                double lowY = base.CalculateYPoint(min, max, bar.Low, chartElementHeight);
                double closeY = base.CalculateYPoint(min, max, bar.Close, chartElementHeight);
                double openY = base.CalculateYPoint(min, max, bar.Open, chartElementHeight);
				
				highY = Math.Min(highY, chartElementHeight - height * 2);
				lowY = Math.Min(lowY, chartElementHeight - height * 2);
				closeY = Math.Min(closeY, chartElementHeight - height * 2);
				openY = Math.Min(openY, chartElementHeight - height * 2);
				
				if (lowY > 0 && highY < chartElementHeight - height * 3)
				{
                	if (bar.Close > bar.Open)
	                {
    	               dc.DrawLine(base.UpWickPen, new Point(x, lowY), new Point(x, highY));
    	               dc.DrawRectangle(base.UpBarBrush, base.UpWickPen,  new Rect(x - base.BarSpace / 2, closeY, this.BarSpace, openY - closeY));
    	            }
    	            else
    	            {
    	                dc.DrawLine(base.UpWickPen, new Point(x, lowY), new Point(x, highY));
    	                dc.DrawRectangle(base.DownBarBrush, base.DownWickPen, new Rect(x - base.BarSpace / 2, openY, base.BarSpace, closeY - openY));
    	            }
            	}
                
                //draw the levels
                for (int j = 0; j < barType.Items[i].Bars.Length; j++)
                {
                    if (barType.Items[i].Bars[j] != null)
                    {
                        double price = barType.Items[i].Max - (j * Math.Max(barType.TickSize, bars.Instrument.InstrumentDefination.TickSize));
                        double y = CalculateYPoint(min, max, barType.Items[i].Bars[j].Price, chartElementHeight);
                        
                        if (y < 0 || y > chartElementHeight - height * 2)
                        	continue;
                        
                        if (!this.ShowDelta)
                        {
							GlyphRun txt = Common.Globals.GetGlyphRun(barType.Items[i].Bars[j].Delta.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x - this.BarWidth / 4, y + height / 4), true);
                        	dc.DrawGlyphRun(barType.Items[i].Bars[j].Delta >= 0 ? base.UpBarBrush : base.DownBarBrush, txt);
                        	
                        	if (this.ShowProfile)
                        	{
                        		double pWidth = (double)barType.Items[i].Bars[j].TotalVolume / (double)barType.MaxPOCVolume * (this.BarWidth / 2);
                        		dc.DrawRectangle(this.ProfileBrush, null, new Rect(x + base.BarSpace / 2 + 1, y - height / 2, pWidth, height));
                        	}
                        	else
                        	{
                        		txt = Common.Globals.GetGlyphRun(barType.Items[i].Bars[j].TotalVolume.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x + BarWidth / 4, y + height / 4), true);                      
                        		dc.DrawGlyphRun(base.UpWickBrush, txt);
                        	}
                        }
                        else
                        {
                        	GlyphRun txt = Common.Globals.GetGlyphRun(barType.Items[i].Bars[j].BidVolume.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x - this.BarWidth / 4, y + height / 4), true);
                        	dc.DrawGlyphRun(barType.Items[i].Bars[j].Delta >= 0 ? base.DownBarBrush : base.UpBarBrush, txt);
                        
                        	txt = Common.Globals.GetGlyphRun(barType.Items[i].Bars[j].AskVolume.ToString(), Common.Options.FontFamilySource, Common.Options.FontSize, new Point(x + BarWidth / 4, y + height / 4), true);                      
                        	dc.DrawGlyphRun(barType.Items[i].Bars[j].Delta >= 0 ? base.UpBarBrush : base.DownBarBrush, txt);
                        }
                    }
                }
                
                                
                x -= (base.BarWidth + base.BarSpace);
                
                if (this.ShowProfile && x < this.ProfileWidth)
                	break;
            }
            
            
            //draw the cumulative profiles
            if (this.ShowProfile && barType.MainBar != null && barType.MainBar.Bars != null)
			{
				double tickSize = Math.Max(barType.TickSize, bars.Instrument.InstrumentDefination.TickSize);
			
				double minValue = barType.MainBar.CalculateStepPrice(Math.Max(min, barType.MainBar.Min));
			
				double maxValue = barType.MainBar.CalculateStepPrice(Math.Min(max, barType.MainBar.Max));
			
				int minIdx = barType.MainBar.Bars.Length - (int)Math.Round((minValue - barType.MainBar.Min) / tickSize, MidpointRounding.AwayFromZero);
				int maxIdx = barType.MainBar.Bars.Length - 1 - (int)Math.Round((maxValue - barType.MainBar.Min) / tickSize, MidpointRounding.AwayFromZero);
				
				
					
				for (int i = minIdx; i-- > maxIdx;)
				{
					if (barType.MainBar.Bars[i] == null)
						continue;
					
					double y = base.CalculateYPoint(min, max, barType.MainBar.Bars[i].Price, chartElementHeight);
					double pWidth = (double)barType.MainBar.Bars[i].TotalVolume / (double)barType.MainBar.POCVolume * this.ProfileWidth;
					
					dc.DrawRectangle(this.ProfileBrush, null, new Rect(0 , y - height / 2, pWidth, height));
				}
				
				if (this.ShowPOC)
				{
					double y = base.CalculateYPoint(min, max, barType.MainBar.CalculateStepPrice(barType.MainBar.POC), chartElementHeight);
					if (y > 0 && y < chartElementHeight)
					{
						dc.DrawRectangle(null, base.UpWickPen, new Rect(0, y - height / 2, this.ProfileWidth, height));
					}
				}
			}
            
            
        }
        
               
        public bool ShowPOC { get; set; }
        public bool ShowDelta { get; set; }
        public bool ShowProfile { get; set; }
        public bool IsPercentage { get; set; }
        
        private double profileWidth = 100;
        
        public double ProfileWidth
        {
        	get { return profileWidth; }
        	set
        	{
        		profileWidth = Math.Max(20, value);
        	}
        }

        private Brush profileBrush;
        [XmlElement(ElementName = "ProfileBrush", Type = typeof(XamlSerializer<Brush>))]
        public Brush ProfileBrush
        {
            get { return profileBrush; }
            set 
            {
                profileBrush = value;
                if (profileBrush != null && profileBrush.CanFreeze)
                    profileBrush.Freeze();
            }
        }
        
                
        
    }
}